@@ -5,7 +5,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
-static char *id = "@(#) $Id: cpuid.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: cpuid.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 #if defined(__i386__) || defined(__alpha__)
 
@@ -58,6 +58,7 @@ static hwNode *getcpu(hwNode & node,
     hwNode cpu("cpu", hw::processor);
 
     cpu.setBusInfo(cpubusinfo);
+    cpu.addHint("icon", string("cpu"));
     cpu.claim();
 
     return core->addChild(cpu);
@@ -264,6 +265,10 @@ static bool dointel(unsigned long maxi,
     snprintf(buffer, sizeof(buffer), "%d.%d.%d", family, model, stepping);
     cpu->setVersion(buffer);
 
+    
+    if(ecx & (1 << 5))
+      cpu->addCapability("vmx", "CPU virtualization (Vanderpool)");
+
     /* Hyper-Threading Technology */
     if (flags & (1 << 28))
     {
@@ -278,6 +283,7 @@ static bool dointel(unsigned long maxi,
       logicalcpu.setDescription("Logical CPU");
       logicalcpu.addCapability("logical", "Logical CPU");
       logicalcpu.setWidth(cpu->getWidth());
+      logicalcpu.claim();
       cpu->addCapability("ht", "HyperThreading");
 
       if(nr_ht>1)
@@ -287,6 +293,7 @@ static bool dointel(unsigned long maxi,
 	  logicalcpu.setHandle(buff);
           logicalcpu.setPhysId(phys_id, i+1);
           cpu->addChild(logicalcpu);
+          cpu->claim();
         }
     }
 
