@@ -1,4 +1,5 @@
 #include "disk.h"
+#include "partitions.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -6,7 +7,7 @@
 #include <unistd.h>
 //#include <linux/fs.h>
 
-static char *id = "@(#) $Id: disk.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: disk.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 #ifndef BLKROGET
 #define BLKROGET   _IO(0x12,94)	/* get read-only status (0 = read_write) */
@@ -55,6 +56,12 @@ bool scan_disk(hwNode & n)
 
   close(fd);
 
+  if(n.getSize()>=0)
+  {
+    n.addHint("icon", string("disc"));
+    scan_partitions(n);
+  }
+
   (void) &id;			// to avoid warning "id defined but not used"
 
   return true;
