@@ -87,7 +87,7 @@
 #include <stdlib.h>
 #include <sys/mman.h>
 
-static char *id = "@(#) $Id: dmi.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: dmi.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 static int currentcpu = 0;
 
@@ -555,40 +555,44 @@ static string dmi_memory_device_detail(u16 v)
 void dmi_chassis(u8 code, hwNode & n)
 {
   static const char *chassis_type[] = {
-    "", "",		/* 0x00 */
-    "", "",
-    "", "",
-    "desktop", "Desktop Computer",
-    "low-profile", "Low Profile Desktop Computer",
-    "pizzabox", "Pizza Box Computer",
-    "mini-tower", "Mini Tower Computer",
-    "tower", "Tower Computer",
-    "portable", "Portable Computer",
-    "laptop", "Laptop",
-    "notebook", "Notebook",
-    "handheld", "Hand Held Computer",
-    "docking", "Docking Station",
-    "all-in-one", "All In One",
-    "sub-notebook", "Sub Notebook",
-    "space-saving", "Space-saving Computer",
-    "lunchbox", "Lunch Box Computer",
-    "server", "System",
-    "expansion", "Expansion Chassis",
-    "sub", "Sub Chassis",
-    "bus-expansion", "Bus Expansion Chassis",
-    "peripheral", "Peripheral Chassis",
-    "raid", "RAID Chassis",
-    "rackmount", "Rack Mount Chassis",
-    "sealed", "Sealed-case PC",
-    "multi-system", "Multi-system" /* 0x19 */
+    "", "", NULL,		/* 0x00 */
+    "", "", NULL,
+    "", "", NULL,
+    "desktop", "Desktop Computer", "desktopcomputer",
+    "low-profile", "Low Profile Desktop Computer", "desktopcomputer",
+    "pizzabox", "Pizza Box Computer", "pizzabox",
+    "mini-tower", "Mini Tower Computer", "towercomputer",
+    "tower", "Tower Computer", "towercomputer",
+    "portable", "Portable Computer", "laptop",
+    "laptop", "Laptop", "laptop",
+    "notebook", "Notebook", "laptop",
+    "handheld", "Hand Held Computer", "pda",
+    "docking", "Docking Station", NULL,
+    "all-in-one", "All In One", NULL,
+    "sub-notebook", "Sub Notebook", "laptop",
+    "space-saving", "Space-saving Computer", NULL,
+    "lunchbox", "Lunch Box Computer", NULL,
+    "server", "System", "server",
+    "expansion", "Expansion Chassis", NULL,
+    "sub", "Sub Chassis", NULL,
+    "bus-expansion", "Bus Expansion Chassis", NULL,
+    "peripheral", "Peripheral Chassis", NULL,
+    "raid", "RAID Chassis", "md",
+    "rackmount", "Rack Mount Chassis", "rackmount",
+    "sealed", "Sealed-case PC", NULL,
+    "multi-system", "Multi-system", "cluster" /* 0x19 */
   };
 
   if(code <= 0x19)
   {
-    if(n.getDescription()=="") n.setDescription(chassis_type[1+2*code]);
+    if(n.getDescription()=="") n.setDescription(chassis_type[1+3*code]);
 
-    if(code >= 2)
-       n.setConfig("chassis", chassis_type[2*code] );
+    if(code >= 3)
+    {
+       n.setConfig("chassis", chassis_type[3*code] );
+       if(chassis_type[2+3*code])
+         n.addHint("icon", string(chassis_type[2+3*code]));
+    }
   }
 };
 static const char *dmi_processor_family(u8 code)
@@ -947,6 +951,7 @@ static void dmi_table(int fd,
 	//newnode.setPhysId(16 * (data[7] << 8 | data[6]));
 	newnode.setPhysId(dm->handle);
 	newnode.setDescription("BIOS");
+	newnode.addHint("icon", string("chip"));
 	newnode.claim();
 
 	dmi_bios_features(data[13] << 24 | data[12] << 16 | data[11] << 8 |
@@ -987,6 +992,7 @@ static void dmi_table(int fd,
 	  hardwarenode->setSlot(dmi_string(dm, data[0x0A]));
 	  hardwarenode->setHandle(handle);
 	  hardwarenode->setDescription("Motherboard");
+	  hardwarenode->addHint("icon", string("motherboard"));
 	}
 	else
 	{
@@ -1039,6 +1045,7 @@ static void dmi_table(int fd,
         newnode.setBusInfo(cpubusinfo(currentcpu++));
 	newnode.setSlot(dmi_string(dm, data[4]));
 	newnode.setDescription("CPU");
+	newnode.addHint("icon", string("cpu"));
 	newnode.setProduct(dmi_processor_family(data[6]));
 	newnode.setVersion(dmi_string(dm, data[0x10]));
 	newnode.setVendor(dmi_string(dm, data[7]));
@@ -1132,6 +1139,7 @@ static void dmi_table(int fd,
 	  clock = 1000000000 / data[6];	// convert value from ns to Hz
 	newnode.setClock(clock);
 	newnode.setDescription(dmi_decode_ram(data[8] << 8 | data[7]));
+	newnode.addHint("icon", string("memory"));
 	// installed size
 	switch (data[9] & 0x7F)
 	{
@@ -1286,6 +1294,7 @@ static void dmi_table(int fd,
 	case 0x03:
 	  description = "System Memory";
           newnode.claim();
+	  newnode.addHint("icon", string("memory"));
 	  break;
 	case 0x04:
 	  description = "Video Memory";
@@ -1298,9 +1307,11 @@ static void dmi_table(int fd,
 	  break;
 	case 0x07:
 	  description = "Cache Memory";
+	  newnode.addHint("icon", string("memory"));
 	  break;
 	default:
 	  description = "Generic Memory";
+	  newnode.addHint("icon", string("memory"));
 	}
 
 	newnode.setHandle(handle);
@@ -1328,6 +1339,7 @@ static void dmi_table(int fd,
 	char bits[10];
 	string arrayhandle;
         newnode.setDescription("empty memory bank");
+	newnode.addHint("icon", string("memory"));
 	arrayhandle = dmi_handle(data[5] << 8 | data[4]);
 	strcpy(bits, "");
 	// total width
@@ -1405,6 +1417,7 @@ static void dmi_table(int fd,
 	  hwNode ramnode("memory",
 			 hw::memory);
 	  ramnode.addChild(newnode);
+          ramnode.addHint("icon", string("memory"));
 	  hardwarenode->addChild(ramnode);
 	}
       }
@@ -1478,6 +1491,7 @@ static void dmi_table(int fd,
       {
         hwNode batt("battery", hw::power);
 
+        batt.addHint("icon", string("battery"));
         batt.claim();
 	batt.setHandle(handle);
 	batt.setVendor(dmi_string(dm, data[0x05]));
