@@ -1,13 +1,17 @@
 #include "hw.h"
 #include "osutils.h"
+#include "version.h"
+#include "options.h"
 #include <vector>
 #include <map>
+#include <sstream>
+#include <unistd.h>
 #include <stdio.h>
 #include <ctype.h>
 
 using namespace hw;
 
-static char *id = "@(#) $Id: hw.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: hw.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 struct hwNode_i
 {
@@ -28,6 +32,7 @@ struct hwNode_i
     map < string, string > features_descriptions;
     vector < resource > resources;
     map < string, string > config;
+    map < string, value > hints;
 };
 
 string hw::strip(const string & s)
@@ -1085,6 +1090,10 @@ void hwNode::merge(const hwNode & node)
   for (map < string, string >::iterator i = node.This->config.begin();
        i != node.This->config.end(); i++)
     setConfig(i->first, i->second);
+
+  for (map < string, value >::iterator i = node.This->hints.begin();
+       i != node.This->hints.end(); i++)
+    addHint(i->first, i->second);
 }
 
 void hwNode::addResource(const resource & r)
@@ -1135,6 +1144,306 @@ unsigned int hwNode::getWidth() const
     return 0;
 }
 
+void hwNode::addHint(const string & id, const value & v)
+{
+  if(This)
+    This->hints[id] = v;
+}
+
+value hwNode::getHint(const string & id) const
+{
+  if(!This)
+    return value();
+
+  return This->hints[id];
+}
+
+string hwNode::asXML(unsigned level)
+{
+  vector < string > config;
+  vector < string > resources;
+  ostringstream out;
+
+  if(!This) return "";
+
+  config = getConfigValues("\" value=\"");
+  resources = getResources("\" value=\"");
+
+  if (level == 0)
+  {
+    out << "<?xml version=\"1.0\" standalone=\"yes\" ?>" << endl;
+    out << "<!-- generated by lshw-" << getpackageversion() << " -->" <<
+#if defined(__GNUC__) && defined(__VERSION__)
+    endl << "<!-- GCC " << __VERSION__ << " -->" <<
+#endif
+      endl;
+    if (geteuid() != 0)
+      out << "<!-- WARNING: not running as root -->" << endl;
+  }
+
+  if(visible(getClassName()))
+  {
+    out << spaces(2*level);
+    out << "<node id=\"" << getId() << "\"";
+    if (disabled())
+      out << " disabled=\"true\"";
+    if (claimed())
+      out << " claimed=\"true\"";
+
+    out << " class=\"" << getClassName() << "\"";
+    out << " handle=\"" << getHandle() << "\"";
+    out << ">" << endl;
+
+    if (getDescription() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<description>";
+      out << escape(getDescription());
+      out << "</description>";
+      out << endl;
+    }
+
+    if (getProduct() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<product>";
+      out << escape(getProduct());
+      out << "</product>";
+      out << endl;
+    }
+
+    if (getVendor() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<vendor>";
+      out << escape(getVendor());
+      out << "</vendor>";
+      out << endl;
+    }
+
+    if (getPhysId() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<physid>";
+      out << getPhysId();
+      out << "</physid>";
+      out << endl;
+    }
+
+    if (getBusInfo() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<businfo>";
+      out << escape(getBusInfo());
+      out << "</businfo>";
+      out << endl;
+    }
+
+    if (getLogicalName() != "")
+    {
+      vector<string> logicalnames = getLogicalNames();
+
+      for(unsigned int i = 0; i<logicalnames.size(); i++)
+      {
+      	out << spaces(2*level+1);
+        out << "<logicalname>";
+      	out << logicalnames[i];
+        out << "</logicalname>";
+      	out << endl;
+      }
+    }
+
+    if (getDev() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<dev>";
+      out << escape(getDev());
+      out << "</dev>";
+      out << endl;
+    }
+
+    if (getVersion() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<version>";
+      out << escape(getVersion());
+      out << "</version>";
+      out << endl;
+    }
+
+    if (getSerial() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<serial>";
+      out << escape(getSerial());
+      out << "</serial>";
+      out << endl;
+    }
+  
+    if (getSlot() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<slot>";
+      out << escape(getSlot());
+      out << "</slot>";
+      out << endl;
+    }
+  
+    if (getSize() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<size";
+      switch (getClass())
+      {
+      case hw::memory:
+      case hw::address:
+      case hw::storage:
+      case hw::disk:
+      case hw::display:
+        out << " units=\"bytes\"";
+        break;
+  
+      case hw::processor:
+      case hw::bus:
+      case hw::system:
+        out << " units=\"Hz\"";
+        break;
+  
+      case hw::power:
+        out << " units=\"mWh\"";
+        break;
+  
+      default:
+        out << "";
+      }
+      out << ">";
+      out << getSize();
+      out << "</size>";
+      out << endl;
+    }
+  
+    if (getCapacity() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<capacity";
+      switch (getClass())
+      {
+      case hw::memory:
+      case hw::address:
+      case hw::storage:
+      case hw::disk:
+        out << " units=\"bytes\"";
+        break;
+  
+      case hw::processor:
+      case hw::bus:
+      case hw::system:
+        out << " units=\"Hz\"";
+        break;
+  
+      case hw::power:
+        out << " units=\"mWh\"";
+        break;
+  
+      default:
+        out << "";
+      }
+      out << ">";
+      out << getCapacity();
+      out << "</capacity>";
+      out << endl;
+    }
+
+    if (getWidth() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<width units=\"bits\">";
+      out << getWidth();
+      out << "</width>";
+      out << endl;
+    }
+
+    if (getClock() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<clock units=\"Hz\">";
+      out << getClock();
+      out << "</clock>";
+      out << endl;
+    }
+
+    if (config.size() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<configuration>" << endl;
+      for (unsigned int j = 0; j < config.size(); j++)
+      {
+        out << spaces(2*level+2);
+        out << "<setting id=\"" << escape(config[j]) << "\" />";
+        out << endl;
+      }
+      out << spaces(2*level+1);
+      out << "</configuration>" << endl;
+    }
+    config.clear();
+  
+    splitlines(getCapabilities(), config, ' ');
+    if (config.size() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<capabilities>" << endl;
+      for (unsigned int j = 0; j < config.size(); j++)
+      {
+        out << spaces(2*level+2);
+        out << "<capability id=\"" << escape(config[j]);
+        if (getCapabilityDescription(config[j]) == "")
+        {
+	  out << "\" />";
+        }
+        else
+        {
+	  out << "\" >";
+	  out << escape(getCapabilityDescription(config[j]));
+	  out << "</capability>";
+        }
+        out << endl;
+      }
+      out << spaces(2*level+1);
+      out << "</capabilities>" << endl;
+    }
+    config.clear();
+
+    if (resources.size() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<resources>" << endl;
+      for (unsigned int j = 0; j < resources.size(); j++)
+      {
+        out << spaces(2*level+2);
+        out << "<resource type=\"" << escape(resources[j]) << "\" />";
+        out << endl;
+      }
+      out << spaces(2*level+1);
+      out << "</resources>" << endl;
+    }
+    resources.clear();
+  }
+
+  for (unsigned int i = 0; i < countChildren(); i++)
+  {
+    out << getChild(i)->asXML(visible(getClassName()) ? level + 1 : 1);
+  }
+
+  if(visible(getClassName()))
+  {
+    out << spaces(2*level);
+    out << "</node>" << endl;
+  }
+
+
+  return out.str();
+}
+
 struct hw::resource_i
 {
   hw::hwResourceType type;
@@ -1350,3 +1659,140 @@ bool resource::operator == (const resource & r)
 	 default:return false;
        }
      }
+
+struct hw::value_i
+{
+  hw::hwValueType type;
+
+    long long ll;
+    string s;
+    bool b;
+
+  int refcount;
+};
+
+value::value()
+{
+  This = new hw::value_i;
+
+  if(This)
+  {
+    This->type = nil;
+    This->refcount = 1;
+  }
+}
+
+value::~value()
+{
+  if(This)
+  {
+    This->refcount--;
+    if(This->refcount<=0)
+      delete This;
+    This = NULL;
+  }
+}
+
+value::value(long long ll)
+{
+  This = new hw::value_i;
+
+  if(This)
+  {
+    This->type = integer;
+    This->ll = ll;
+    This->refcount = 1;
+  }
+}
+
+value::value(const string & s)
+{
+  This = new hw::value_i;
+
+  if(This)
+  {
+    This->type = text;
+    This->s = s;
+    This->refcount = 1;
+  }
+}
+
+value::value(const value & v)
+{
+  This = v.This;
+
+  if(This)
+  {
+    This->refcount++;
+  }
+}
+
+value & value::operator=(const value & v)
+{
+  if(v.This == This) return *this;
+
+  if(This)
+  {
+    This->refcount--;
+    if(This->refcount<=0)
+      delete This;
+  }
+
+  This = v.This;
+  if(This)
+    This->refcount++;
+
+  return *this;
+}
+
+bool value::operator==(const value & v) const
+{
+  if(v.This == This) return true;
+
+  if(!v.This || !This) return false;
+
+  if(v.This->type != This->type) return false;
+
+  switch(This->type)
+  {
+    case hw::integer:
+      return This->ll == v.This->ll;
+    case hw::text:
+      return This->s == v.This->s;
+    case hw::boolean:
+      return This->b == v.This->b;
+    case hw::nil:
+      return true;
+  };
+
+  return false;
+}
+
+
+string value::asString() const
+{
+  if(!This) return "";
+
+  switch(This->type)
+  {
+    case hw::integer:
+      return "0";
+    case hw::text:
+      return This->s;
+    case hw::boolean:
+      return This->b?"true":"false";
+    case hw::nil:
+      return "(nil)";
+  };
+
+  return "";
+}
+
+bool value::defined() const
+{
+  if(!This) return false;
+  if(This->type==nil) return false;
+
+  return true;
+}
+
