@@ -28,8 +28,9 @@ typedef enum {
 	generic} hwClass;
 
 typedef enum { none, iomem, ioport, mem, irq, dma } hwResourceType;
+typedef enum { nil, boolean, integer, text } hwValueType;
 
-string hw::strip(const string &);
+string strip(const string &);
 
 class resource
 {
@@ -55,6 +56,27 @@ class resource
 
 };
 
+class value
+{
+  public:
+
+        value();
+        ~value();
+        value(const value &);
+        value(long long);
+        value(const string &);
+	value & operator =(const value &);
+
+	bool operator ==(const value &) const;
+
+        string asString() const;
+        bool defined() const;
+
+  private:
+	struct value_i * This;
+
+};
+
 } // namespace hw
 
 class hwNode
@@ -168,11 +190,14 @@ class hwNode
 	bool usesResource(const hw::resource &) const;
 	vector<string> getResources(const string & separator = "") const;
 
-        void setSpeed(double s, const string & units = "");
+	void addHint(const string &, const hw::value &);
+	hw::value getHint(const string &) const;
 
 	void merge(const hwNode & node);
 
         void fixInconsistencies();
+
+	string asXML(unsigned level = 0);
   private:
 
 	void setId(const string & id);
