@@ -1,3 +1,16 @@
+/*
+ *  ide.cc
+ *
+ *  This scan tries to detect all IDE interfaces in the system by scanning
+ *  /proc/ide and looking for /proc/ide/xxx/channel
+ *  It then tries to find the parent device for this interface (on 2.4 kernels,
+ *  /proc/ide/xxx/identify contains useful information for PCI devices), other-
+ *  wise, guessParent() is used.
+ *  Each IDE-connected device is scanned and more information is gathered
+ *  by calling scan_disk() and scan_cdrom(), as appropriate.
+ */
+
+
 #include "cpuinfo.h"
 #include "osutils.h"
 #include "cdrom.h"
@@ -16,7 +29,7 @@
 #include <linux/hdreg.h>
 #include <regex.h>
 
-static char *id = "@(#) $Id: ide.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: ide.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 #define PROC_IDE "/proc/ide"
 
@@ -469,11 +482,12 @@ bool scan_ide(hwNode & n)
 
 	      if (candidate)
               {
+                parent = candidate;
 	        candidate->merge(*ide.getChild(k));
                 break;
               }
 	    }
-	  //n.addChild(ide);
+	  if(!parent) n.addChild(ide);
 	}
       }
 
