@@ -36,11 +36,12 @@
 #include "sysfs.h"
 #include "display.h"
 #include "parisc.h"
+#include "cpufreq.h"
 
 #include <unistd.h>
 #include <stdio.h>
 
-static char *id = "@(#) $Id: main.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: main.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 bool scan_system(hwNode & system)
 {
@@ -102,6 +103,9 @@ bool scan_system(hwNode & system)
     status("Display");
     if (enabled("display"))
       scan_display(computer);
+    status("CPUFreq");
+    if (enabled("cpufreq"))
+      scan_cpufreq(computer);
     status("");
 
     if (computer.getDescription() == "")
