@@ -9,12 +9,13 @@
 #include <regex.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <errno.h>
 #ifndef MINOR
 #include <linux/kdev_t.h>
 #endif
 
 static char *id =
-  "@(#) $Id: osutils.cc 951 2005-02-04 13:51:03Z ezix $";
+  "@(#) $Id: osutils.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 using namespace std;
 
@@ -165,6 +166,9 @@ int selectlink(const struct dirent *d)
   if (d->d_name[0] == '.')
     return 0;
 
+  if (lstat(d->d_name, &buf) != 0)
+    return 0;
+
   return S_ISLNK(buf.st_mode);
 }
 
@@ -335,3 +339,137 @@ string readlink(const string & path)
   else
     return path;
 }
+
+string realpath(const string & path)
+{
+  char buffer[PATH_MAX+1];
+
+  memset(buffer, 0, sizeof(buffer));
+  if(realpath(path.c_str(), buffer))
+    return string(buffer);
+  else
+    return path;
+}
+
+string spaces(unsigned int count, const string & space)
+{
+  string result = "";
+  while (count-- > 0)
+    result += space;
+
+  return result;
+}
+
+string escape(const string & s)
+{
+  string result = "";
+
+  for (unsigned int i = 0; i < s.length(); i++)
+    switch (s[i])
+    {
+    case '<':
+      result += "&lt;";
+      break;
+    case '>':
+      result += "&gt;";
+      break;
+    case '&':
+      result += "&amp;";
+      break;
+    default:
+      result += s[i];
+    }
+
+  return result;
+}
+
+unsigned short be_short(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((__uint16_t)(p[0]) << 8) +
+	(__uint16_t)p[1];
+}
+
+unsigned short le_short(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((__uint16_t)(p[1]) << 8) +
+	(__uint16_t)p[0];
+}
+
+unsigned long be_long(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((__uint32_t)(p[0]) << 24) +
+	((__uint32_t)(p[1]) << 16) +
+	((__uint32_t)(p[2]) << 8) +
+	(__uint32_t)p[3];
+}
+
+unsigned long le_long(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((__uint32_t)(p[3]) << 24) +
+	((__uint32_t)(p[2]) << 16) +
+	((__uint32_t)(p[1]) << 8) +
+	(__uint32_t)p[0];
+
+}
+
+unsigned long long be_longlong(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((unsigned long long)(p[0]) << 56) +
+	((unsigned long long)(p[1]) << 48) +
+	((unsigned long long)(p[2]) << 40) +
+	((unsigned long long)(p[3]) << 32) +
+	((unsigned long long)(p[4]) << 24) +
+	((unsigned long long)(p[5]) << 16) +
+	((unsigned long long)(p[6]) << 8) +
+	(unsigned long long)p[7];
+}
+
+unsigned long long le_longlong(void * from)
+{
+  __uint8_t *p = (__uint8_t*)from;
+
+  return ((unsigned long long)(p[7]) << 56) +
+	((unsigned long long)(p[6]) << 48) +
+	((unsigned long long)(p[5]) << 40) +
+	((unsigned long long)(p[4]) << 32) +
+	((unsigned long long)(p[3]) << 24) +
+	((unsigned long long)(p[2]) << 16) +
+	((unsigned long long)(p[1]) << 8) +
+	(unsigned long long)p[0];
+}
+
+int open_dev(dev_t dev, const string & name)
+{
+  static char *paths[] = {
+    "/usr/tmp", "/var/tmp", "/var/run", "/dev", "/tmp", NULL
+  };
+  char **p, fn[64];
+  int fd;
+
+  for (p = paths; *p; p++)
+  {
+    if(name=="")
+      snprintf(fn, sizeof(fn), "%s/lshw-%d", *p, getpid());
+    else
+      snprintf(fn, sizeof(fn), "%s", name.c_str());
+    if ((mknod(fn, (S_IFCHR | S_IREAD), dev) == 0) || (errno = EEXIST))
+    {
+      fd = open(fn, O_RDONLY);
+      if(name=="") unlink(fn);
+      if (fd >= 0)
+	return fd;
+    }
+  }
+  return -1;
+}				/* open_dev */
+
