@@ -6,7 +6,7 @@
 #include <unistd.h>
 #include <stdio.h>
 
-static char *id = "@(#) $Id: pci.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: pci.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 #define PROC_BUS_PCI "/proc/bus/pci"
 #define PCIID_PATH "/usr/share/lshw/pci.ids:/usr/local/share/pci.ids:/usr/share/pci.ids:/etc/pci.ids:/usr/share/hwdata/pci.ids:/usr/share/misc/pci.ids"
@@ -97,6 +97,7 @@ static char *id = "@(#) $Id: pci.cc 951 2005-02-04 13:51:03Z ezix $";
 #define PCI_BASE_CLASS_COMMUNICATION	0x07
 #define PCI_CLASS_COMMUNICATION_SERIAL	0x0700
 #define PCI_CLASS_COMMUNICATION_PARALLEL 0x0701
+#define PCI_CLASS_COMMUNICATION_MODEM 0x0703
 #define PCI_CLASS_COMMUNICATION_OTHER	0x0780
 
 #define PCI_BASE_CLASS_SYSTEM		0x08
@@ -146,6 +147,13 @@ static char *id = "@(#) $Id: pci.cc 951 2005-02-04 13:51:03Z ezix $";
 #define  PCI_BASE_ADDRESS_MEM_MASK      (~0x0fUL)
 #define  PCI_BASE_ADDRESS_IO_MASK       (~0x03UL)
 
+#define PCI_SUBSYSTEM_VENDOR_ID 0x2c
+#define PCI_SUBSYSTEM_ID        0x2e
+
+#define PCI_CB_SUBSYSTEM_VENDOR_ID 0x40
+#define PCI_CB_SUBSYSTEM_ID     0x42
+
+
 typedef unsigned long long pciaddr_t;
 typedef enum
 { pcidevice,
@@ -594,6 +602,23 @@ static bool scan_resources(hwNode & n,
   return true;
 }
 
+static void addHints(hwNode & n,
+		long _vendor,
+		long _device,
+		long _subvendor,
+		long _subdevice,
+		long _class)
+{
+  n.addHint("pci.vendor", _vendor);
+  n.addHint("pci.device", _device);
+  if(_subvendor && (_subvendor != 0xffff))
+  {
+    n.addHint("pci.subvendor", _subvendor);
+    n.addHint("pci.subdevice", _subdevice);
+  }
+  n.addHint("pci.class", _class);
+}
+
 bool scan_pci(hwNode & n)
 {
   FILE *f;
@@ -668,12 +693,29 @@ bool scan_pci(hwNode & n)
       u_int16_t status = get_conf_word(d, PCI_STATUS);
       u_int8_t progif = get_conf_byte(d, PCI_CLASS_PROG);
       u_int8_t rev = get_conf_byte(d, PCI_REVISION_ID);
+      u_int8_t htype = get_conf_byte(d, PCI_HEADER_TYPE) & 0x7f;
+      u_int16_t subsys_v = 0, subsys_d = 0;
 
       char revision[10];
       snprintf(revision, sizeof(revision), "%02x", rev);
       string moredescription = get_class_description(dclass, progif);
       string drivername = hw::strip(driver);
 
+      switch (htype)
+      {
+        case PCI_HEADER_TYPE_NORMAL:
+          subsys_v = get_conf_word(d, PCI_SUBSYSTEM_VENDOR_ID);
+          subsys_d = get_conf_word(d, PCI_SUBSYSTEM_ID);
+          break;
+        case PCI_HEADER_TYPE_BRIDGE:
+          subsys_v = subsys_d = 0;
+          break;
+        case PCI_HEADER_TYPE_CARDBUS:
+          subsys_v = get_conf_word(d, PCI_CB_SUBSYSTEM_VENDOR_ID);
+          subsys_d = get_conf_word(d, PCI_CB_SUBSYSTEM_ID);
+          break;
+      }
+
       if (dclass == PCI_CLASS_BRIDGE_HOST)
       {
 	hwNode host("pci",
@@ -684,6 +726,7 @@ bool scan_pci(hwNode & n)
 	host.setProduct(get_device_description(d.vendor_id, d.device_id));
 	host.setHandle(pci_bushandle(d.bus));
 	host.setVersion(revision);
+        addHints(host, d.vendor_id, d.device_id, subsys_v, subsys_d, dclass);
 	host.claim();
 	host.setBusInfo(businfo);
 	if (d.size[0] > 0)
@@ -712,29 +755,46 @@ bool scan_pci(hwNode & n)
       {
 	hw::hwClass deviceclass = hw::generic;
 	string devicename = "generic";
+        string deviceicon = "";
 
 	switch (dclass >> 8)
 	{
 	case PCI_BASE_CLASS_STORAGE:
 	  deviceclass = hw::storage;
+          deviceicon = "disc";
+          if(dclass == PCI_CLASS_STORAGE_SCSI)
+            deviceicon = "scsi";
+          if(dclass == PCI_CLASS_STORAGE_RAID)
+            deviceicon = "raid";
 	  break;
 	case PCI_BASE_CLASS_NETWORK:
 	  deviceclass = hw::network;
+          deviceicon = "network";
 	  break;
 	case PCI_BASE_CLASS_MEMORY:
 	  deviceclass = hw::memory;
+          deviceicon = "memory";
 	  break;
 	case PCI_BASE_CLASS_BRIDGE:
 	  deviceclass = hw::bridge;
 	  break;
 	case PCI_BASE_CLASS_MULTIMEDIA:
 	  deviceclass = hw::multimedia;
+          if(dclass == PCI_CLASS_MULTIMEDIA_AUDIO)
+            deviceicon = "audio";
 	  break;
 	case PCI_BASE_CLASS_DISPLAY:
 	  deviceclass = hw::display;
+          deviceicon = "display";
 	  break;
 	case PCI_BASE_CLASS_COMMUNICATION:
 	  deviceclass = hw::communication;
+          if(dclass == PCI_CLASS_COMMUNICATION_SERIAL)
+            deviceicon = "serial";
+          if(dclass == PCI_CLASS_COMMUNICATION_PARALLEL)
+            deviceicon = "parallel";
+          if(dclass == PCI_CLASS_COMMUNICATION_MODEM)
+            deviceicon = "modem";
 	  break;
 	case PCI_BASE_CLASS_SYSTEM:
 	  deviceclass = hw::system;
@@ -747,6 +807,10 @@ bool scan_pci(hwNode & n)
 	  break;
 	case PCI_BASE_CLASS_SERIAL:
 	  deviceclass = hw::bus;
+          if(dclass == PCI_CLASS_SERIAL_USB)
+            deviceicon = "usb";
+          if(dclass == PCI_CLASS_SERIAL_FIREWIRE)
+            deviceicon = "firewire";
 	  break;
 	}
 
@@ -755,8 +819,10 @@ bool scan_pci(hwNode & n)
 
 	if (device)
 	{
+	  if(deviceicon != "") device->addHint("icon", deviceicon);
 	  device->setBusInfo(businfo);
 	  device->setPhysId(PCI_SLOT(dfn & 0xff), PCI_FUNC(dfn & 0xff));
+          addHints(*device, d.vendor_id, d.device_id, subsys_v, subsys_d, dclass);
 
 	  if (deviceclass == hw::bridge || deviceclass == hw::storage)
 	    device->addCapability(devicename);
@@ -822,7 +888,7 @@ bool scan_pci(hwNode & n)
 	  device->describeCapability("vga", "VGA graphical framebuffer");
 	  device->describeCapability("pcmcia", "PC-Card (PCMCIA)");
 	  device->describeCapability("generic", "Generic interface");
-	  device->describeCapability("ohci", "Open Host Controller Interface (USB1)");
+	  device->describeCapability("ohci", "Open Host Controller Interface");
 	  device->describeCapability("uhci",
 				  "Universal Host Controller Interface (USB1)");
 	  device->describeCapability("ehci",
