@@ -3,6 +3,7 @@
 #include "disk.h"
 #include "osutils.h"
 #include "heuristics.h"
+#include "sysfs.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -19,7 +20,7 @@
 #include <string>
 #include <map>
 
-static char *id = "@(#) $Id: scsi.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: scsi.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 #define SG_X "/dev/sg%d"
 #define SG_MAJOR 21
@@ -191,6 +192,7 @@ static const char *devices[] = {
   "/dev/cdrom", "/dev/cdrom0", "/dev/cdrom1", "/dev/cdrom2",
   "/dev/cdwriter", "/dev/cdwriter0", "/dev/cdwriter1", "/dev/cdwriter2",
   "/dev/dvd", "/dev/dvd0", "/dev/dvd1", "/dev/dvd2",
+  "/dev/dvdwriter", "/dev/dvdwriter0", "/dev/dvdwriter1", "/dev/dvdwriter2",
   "/dev/st0", "/dev/st1", "/dev/st2", "/dev/st3", "/dev/st4", "/dev/st5",
   "/dev/nst0", "/dev/nst1", "/dev/nst2", "/dev/nst3", "/dev/nst4",
   "/dev/nst5",
@@ -351,7 +353,7 @@ static bool do_modesense(int sg_fd,
 
   res =
     sg_err_category(io_hdr.status, io_hdr.host_status, io_hdr.driver_status,
-		    io_hdr.sbp, io_hdr.sb_len_wr);
+		    (unsigned char*)io_hdr.sbp, io_hdr.sb_len_wr);
   switch (res)
   {
   case SG_ERR_CAT_CLEAN:
@@ -400,7 +402,7 @@ static bool do_inq(int sg_fd,
 
   res =
     sg_err_category(io_hdr.status, io_hdr.host_status, io_hdr.driver_status,
-		    io_hdr.sbp, io_hdr.sb_len_wr);
+		    (unsigned char*)io_hdr.sbp, io_hdr.sb_len_wr);
   switch (res)
   {
   case SG_ERR_CAT_CLEAN:
@@ -473,8 +475,8 @@ static bool do_inquiry(int sg_fd,
   if (do_inq(sg_fd, 0, 1, 0x80, rsp_buff, MX_ALLOC_LEN, 0))
   {
     len = rsp_buff[3];
-    if ((len > 0) && (rsp_buff[0] > ' '))
-      node.setSerial(string(rsp_buff + 4, len));
+    if (len > 0)
+      node.setSerial(hw::strip(string(rsp_buff + 4, len)));
   }
 
   memset(rsp_buff, 0, sizeof(rsp_buff));
@@ -593,12 +595,12 @@ static string scsi_businfo(int host,
 
   if (channel >= 0)
   {
-    snprintf(businfo, sizeof(businfo), ".%d", channel);
+    snprintf(businfo, sizeof(businfo), ":%d", channel);
     result += string(businfo);
 
     if (target >= 0)
     {
-      snprintf(businfo, sizeof(businfo), ":%d", target);
+      snprintf(businfo, sizeof(businfo), ".%d", target);
       result += string(businfo);
 
       if (lun >= 0)
@@ -621,6 +623,15 @@ static string host_logicalname(int i)
   return string(host);
 }
 
+static string host_kname(int i)		// for sysfs
+{
+  char host[20];
+
+  snprintf(host, sizeof(host), "host%d", i);
+
+  return string(host);
+}
+
 static bool atapi(const hwNode & n)
 {
   return n.isCapable("atapi") && (n.countChildren() == 0);
@@ -699,6 +710,7 @@ static bool scan_sg(int sg,
   device.setBusInfo(scsi_businfo(m_id.host_no,
 				 m_id.channel, m_id.scsi_id, m_id.lun));
   device.setPhysId(m_id.channel, m_id.scsi_id, m_id.lun);
+  device.addHint("bus.icon", string("scsi"));
   find_logicalname(device);
   do_inquiry(fd, device);
   if ((m_id.scsi_type == 4) || (m_id.scsi_type == 5))
@@ -793,6 +805,13 @@ static bool scan_hosts(hwNode & node)
 	  hwNode *controller =
 	    node.findChildByLogicalName(host_logicalname(number));
 
+	  if (!controller)
+          {
+            string parentbusinfo = sysfs_getbusinfo(sysfs::entry::byClass("scsi_host", host_kname(number)));
+
+	    controller = node.findChildByBusInfo(parentbusinfo);
+	  }
+
 	  if (!controller)
 	  {
 	    controller = node.addChild(hwNode::hwNode("scsi", hw::storage));
@@ -805,6 +824,7 @@ static bool scan_hosts(hwNode & node)
 
 	  if (controller)
 	  {
+            controller->setLogicalName(host_logicalname(number));
 	    controller->setConfig(string("driver"),
 				  string(namelist[i]->d_name));
             controller->setHandle(scsi_handle(number));
