@@ -63,7 +63,7 @@ static map<u_int32_t,string> usbproducts;
 
 #define PRODID(x, y) ((x << 16) + y)
 
-static char *id = "@(#) $Id: usb.cc 951 2005-02-04 13:51:03Z ezix $";
+static char *id = "@(#) $Id: usb.cc 1187 2005-10-13 21:33:55Z ezix $";
 
 static string usbhost(unsigned bus)
 {
@@ -108,6 +108,8 @@ static bool addUSBChild(hwNode & n, hwNode & device, unsigned bus, unsigned lev,
 {
   hwNode * parent = NULL;
 
+  device.addHint("bus.icon", string("usb"));
+
   if(prnt>0) parent = n.findChildByHandle(usbhandle(bus, lev-1, prnt));
   if(parent)
   {
@@ -192,6 +194,7 @@ static bool setUSBClass(hwNode & device, unsigned cls, unsigned sub, unsigned pr
     case USB_CLASS_PRINTER:
       device.setClass(hw::printer);
       device.setDescription("Printer");
+      device.addHint("icon", string("printer"));
       if(sub==USB_SC_PRINTER)
       {
         switch(prot)
@@ -249,6 +252,7 @@ static bool setUSBClass(hwNode & device, unsigned cls, unsigned sub, unsigned pr
       {
         device.setDescription("Bluetooth wireless interface");
         device.addCapability("bluetooth", "Bluetooth wireless radio");
+	device.addHint("icon", string("bluetooth"));
       }
       break;
     default:
@@ -264,6 +268,8 @@ static bool describeUSB(hwNode & device, unsigned vendor, unsigned prodid)
   if(usbvendors.find(vendor)==usbvendors.end()) return false;
 
   device.setVendor(usbvendors[vendor]);
+  device.addHint("usb.idVendor", vendor);
+  device.addHint("usb.idProduct", prodid);
 
   if(usbproducts.find(PRODID(vendor, prodid))!=usbproducts.end())
     device.setProduct(usbproducts[PRODID(vendor, prodid)]);
@@ -412,6 +418,9 @@ bool scan_usb(hwNode & n)
               device.describeCapability("usb-1.00", "USB 1.0");
               device.describeCapability("usb-1.10", "USB 1.1");
               device.describeCapability("usb-2.00", "USB 2.0");
+              device.addHint("usb.bDeviceClass", cls);
+              device.addHint("usb.bDeviceSubClass", sub);
+              device.addHint("usb.bDeviceProtocol", prot);
             }
             break;
           case 'P':
